# environment variable names
API_TOKEN = "CASTOR_DOMO_API_TOKEN"  # noqa: S105
BASE_URL = "CASTOR_DOMO_BASE_URL"
CLIENT_ID = "CASTOR_DOMO_CLIENT_ID"
CLOUD_ID = "CLOUD_ID"
DEVELOPER_TOKEN = "CASTOR_DOMO_DEVELOPER_TOKEN"  # noqa: S105
